// Bezeichnung: Funktionsauswahl
// Autor      : Andreas Brotzmann, Peter Weigel

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import pefas.sort.*;

class ControlPanel extends BorderPanel implements ActionListener, AdjustmentListener, ItemListener, Runnable {
  protected Panel northpanel;			//panel, das oben dargestellt wird
  protected Panel southpanel;			//das gleiche fr unten (Button Hilfe+Beenden)
  protected ptfield field;			  //Das Feld/Panel ber das die Sortierung visualisiert wird
  protected Choice sortsel;			  //hiermit wird der Sortieralgorithmus gewhlt
  protected Choice variantsel;	  //hiermit wird die Variante gewhlt
  protected Choice elemsel;			  //Anzahl Elemente whlen
  protected Choice fillsel;			  //Fllart whlen
  protected Scrollbar delayscr;		//Scrollbar zum Einstellen der Verzgerung
  protected Label delaylab;
  protected Thread sortthread;		//in diesem Thread wird spter sortiert
  protected Button startbutton;		//Start/Stop-Button
  protected GroupBox fillpanel;		//enthlt Buttons Aufsteigend, Zufllig...
  protected GroupBox sortpanel;		//enthlt sortsel+start/stop
  protected Label fieldsize;      //Statusanzeige - Feldgre
  protected Label fieldstate;     //Statusanzeige - Feldzustand
  protected Label action;         //Statusanzeige - Aktion
  protected TimeLabel TimeCtrl;   //Statusanzeige - Zeit
  protected final String fsUndef  = "undefiniert";
  protected final String fsSortUp = "aufsteigend";
  protected final String fsSortDn = "absteigend";
  protected final String fsRandom = "zufllig";
  protected final String fsSquare = "quadratisch";
  protected final String fsCross  = "kreuzend";
  protected final String fsSpot   = "fleckig";
  protected final String fsSort   = "sortiert";
  protected final String fsError  = "Fehler!!!"; //Ein Sortieralgorithmus sortiert falsch! (drfte nie auftreten)
  protected final String fsEmpty  =  "leer                  ";
  protected final String acWait   = "";
  protected final String acFill   = "fllen";
  protected final String acSort   = "sortieren";

  public ControlPanel(ptfield field) {
    super();
    this.field = field;
    Button b;
    Label l;
    Panel p1;
    Panel p2;
    setLayout(new BorderLayout(10, 10));

    fillpanel = new GroupBox("Feld fuellen", new BorderLayout(0,10));
     elemsel = new Choice();
     elemsel.add("128");
     elemsel.add("256");
     elemsel.add("512");
     elemsel.add("1024");
     elemsel.add("2048");
     elemsel.add("4096");
     elemsel.add("8192");
     elemsel.addItemListener(this);
     elemsel.select(2); // Index auf 512 setzen
     fillsel = new Choice();
     fillsel.add(fsSortUp);
     fillsel.add(fsSortDn);
     fillsel.add(fsRandom);
     fillsel.add(fsSquare);
     fillsel.add(fsCross);
     fillsel.add(fsSpot);
     fillsel.addItemListener(this);
     fillsel.select(2); // Index fr Random

     fillpanel.add(p1 = new Panel(new GridLayout(0,1)), BorderLayout.WEST);
     fillpanel.add(p2 = new Panel(new GridLayout(0,1)), BorderLayout.CENTER);
     p1.add(new Label("Elemente"));p2.add(elemsel);
     p1.add(new Label("Methode"));p2.add(fillsel);

    sortpanel = new GroupBox("Feld sortieren", new BorderLayout(0,0));
     sortpanel.add(p1 = new Panel(new GridLayout(0,1,10,10)), BorderLayout.NORTH);
     sortsel = new Choice();
     String[] n = sortbucket.getNames();
     for (int i=0;i<n.length;i++) {
       sortsel.add(n[i]);
     }
     sortsel.addItemListener(this);
     sortsel.select(0);
     p1.add(sortsel);
     variantsel = new Choice();
     p1.add(variantsel);
     p1.add(startbutton = new Button("Start"));
     startbutton.addActionListener(this);
     sortpanel.add(delaylab = new Label("Delay: 0 ms"), BorderLayout.CENTER);
     delayscr = new Scrollbar(Scrollbar.HORIZONTAL,0,0,0,500);
     delayscr.addAdjustmentListener(this);
     sortpanel.add(delayscr, BorderLayout.SOUTH);

    GroupBox statepanel = new GroupBox("Status", new BorderLayout(0,0));
     statepanel.add(p1 = new Panel(new GridLayout(0,1,0,0)), BorderLayout.WEST);
     statepanel.add(p2 = new Panel(new GridLayout(0,1,0,0)), BorderLayout.CENTER);
     p1.setFont(new Font("Dialog", 1, 12));
     p2.setFont(new Font("Dialog", 0, 12));
     p1.add(new Label("Feldgre"));p2.add(fieldsize = new Label("fieldsize"));
     p1.add(new Label("Feldzustand"));p2.add(fieldstate = new Label("fieldstate"));
     p1.add(new Label("Aktion"));p2.add(action = new Label("action"));
     p1.add(new Label("Laufzeit"));p2.add(TimeCtrl = new TimeLabel("---.- s", new Color(0,128,0), new Font("Courier", 0, 12)));

     setFieldSize(0);
     setFieldState(fsEmpty);
     setAction(acWait);

    northpanel = new Panel(new BorderLayout(10,10));
     northpanel.add(fillpanel,BorderLayout.NORTH);
     northpanel.add(sortpanel,BorderLayout.CENTER);
     northpanel.add(statepanel,BorderLayout.SOUTH);

    southpanel = new GroupBox("");
     southpanel.add(b = new Button("Information"));
     b.addActionListener(this);
     southpanel.add(b = new Button("Beenden"));
     b.addActionListener(this);

    add(northpanel, BorderLayout.NORTH);
    add(southpanel, BorderLayout.SOUTH);

    itemStateChanged(null); // einige Aktionen auslsen
  }

  protected void abort() {
    if (sortthread != null) {
      field.abort();
      while (sortthread != null) {
        try { Thread.sleep(100); }
        catch (InterruptedException e) {};
      }
    }
  }

  protected void setFieldSize(int s) {
    fieldsize.setText((new Integer(s)).toString()+" Elemente");
  }

  protected void setFieldSize(String s) {
    fieldsize.setText(s);
  }

  protected void setFieldState(String s) {
    fieldstate.setText(s);
    if (s.equals(fsSort)) {
      fillsel.select(fsSortUp);
    } else if ((s.equals(fsUndef)) || (s.equals(fsError))) {
      try { fillsel.select(-1); }
      catch (IllegalArgumentException e) {} //Java 1.3 (Netscape 6) kanns nicht
    } else {
      fillsel.select(s);
    }
  }

  protected void setAction(String s) {
    action.setText(s);
  }

//Hauptroutine fr den sortier-thread
  public void run(){
    // Sortieren vorbereiten (Buttons deaktivieren, ...)
     setAction(acSort);
     setFieldState(fsUndef);
     fillsel.setEnabled(false);
     elemsel.setEnabled(false);
     sortsel.setEnabled(false);
     variantsel.setEnabled(false);
     startbutton.setLabel("Stop");
     TimeCtrl.start();
    // Sortieren beginnen
     try {
       isort sortalg = sortbucket.newInstance(sortsel.getSelectedItem(), variantsel.getSelectedItem());
       sortalg.sort(field);
       if (field.sorted()) {
         setFieldState(fsSort);
       } else {
         setFieldState(fsError);
       }
     } catch (RuntimeException e){};
    // Sortieren nachbereiten (Buttons aktivieren, ...)
     TimeCtrl.stop();
     fillsel.setEnabled(true);
     elemsel.setEnabled(true);
     sortsel.setEnabled(true);
     variantsel.setEnabled(true);
     startbutton.setLabel("Start");
     sortthread = null;
     setAction(acWait);
  }

//Event-Behandlung
  public void itemStateChanged(ItemEvent evt) {
    if ((evt == null) ||
        (evt.getSource() == elemsel) ||
        (evt.getSource() == fillsel)) {
      String fsitem = fillsel.getSelectedItem();
      if (fsitem != null) {
        setAction(acFill);
        setFieldSize(fsUndef);
        setFieldState(fsUndef);
        int len = Integer.parseInt(elemsel.getSelectedItem());
        if (fsitem.equals(fsSortUp)) {
          field.sortedfill_up(len);
        }
        if (fsitem.equals(fsSortDn)) {
          field.sortedfill_dn(len);
        }
        if (fsitem.equals(fsRandom)) {
          field.randomfill(len);
        }
        if (fsitem.equals(fsSquare)) {
          field.squarefill(len);
        }
        if (fsitem.equals(fsCross)) {
          field.crossfill(len);
        }
        if (fsitem.equals(fsSpot)) {
          field.spotfill(len);
        }
        setFieldState(fsitem);
        setAction(acWait);
        setFieldSize(field.length());
      }
    }
    if ((evt == null) || (evt.getSource() == sortsel)) { // Varianten updaten
      String[] v = sortbucket.getVariants(sortsel.getSelectedItem());
      variantsel.removeAll(); // Alle Eintrge lschen
      for (int i=0;i<v.length;i++) { // Neue Eintrge eintragen
        variantsel.add(v[i]);
      }
      variantsel.select(0); // Index auf den ersten Eintrag setzen
    }
  }

  public void actionPerformed(ActionEvent evt) {
    if (evt.getActionCommand().equals("Start")) {
      abort();
      sortthread = new Thread(this);
      sortthread.start();
    }
    if (evt.getActionCommand().equals("Stop")) {
      abort();
    }
    if (evt.getActionCommand().equals("Beenden")) {
      vasa.MainWindow.exit();
    }
    if (evt.getActionCommand().equals("Information")) {
      new AboutBox(vasa.MainWindow);
    }
  }

  public void adjustmentValueChanged(AdjustmentEvent e){
    //if (e.getAdjustable() == delayscr){
      delaylab.setText("Delay: " + ((float)e.getValue())/100 + "ms");
      field.setDelay(e.getValue()*10000);
    //}
  }

}


