// Peter Weigel

// field  (abstract class)

/*
  +-----------------------------------------------------------------------------------------+
  | Basis-Implementierung von "ifield".                                                     |
  +------------+----------------------------------------------------------------------------+
  | name       | field                                                                      |
  | extends    | object                                                                     |
  | implements | ifield                                                                     |
  | type       | abstract class                                                             |
  +------------+----------------------------------------------------------------------------+
  | public             ifield(int n);             // Konstruktor, maximale Feldlnge n      |
  | public    int      maxLength();               // maximale Feldlnge ermitteln           |
  | public    int      length();                  //                                        |
  | public    boolean  lt(int i, int j);          //                                        |
  | public    boolean  gt(int i, int j);          //                                        |
  | public    boolean  eq(int i, int j);          //                                        |
  | public    void     exchange(int i, int j);    //                                        |
  | public    void     move(int i, int j);        //                                        |
  | protected void     moveup(int i, int j);      //                                        |
  | protected void     movedown(int i, int j);    //                                        |
  | protected void     item(int i, ielement ele); // Element setzen                         |
  | protected ielement item(int i);               // Element ermitteln                      |
  | protected void     append(ielement ele);      // Neues Element am Ende hinzufgen       |
  | public    void     delete();                  // Letztes Element lschen                |
  | public    void     clear();                   // Gesamtes Feld leeren                   |
  | public    String   toString();                //                                        |
  +-----------------------------------------------------------------------------------------+
*/

package pefas.field;

import java.util.*;

public abstract class field implements ifield {
  protected ielement[] items;
  protected int        length;

  public field(int n) {                 // Konstruktor
    items  = new ielement[n];           // Maximallnge festlegen, aber
    length = 0;                         // am Anfang ist es trotzdem leer.
  }

  public int maxLength() {              // maximale Feldlnge ermitteln
    return items.length;                // Das kann uns das Feld am besten sagen.
  }

  public int length() {                 // reale Feldlnge ermitteln
    return length;                      // Die Lnge steht immer hier drin.
  }

  public boolean lt(int i, int j) {            // Vergleich zweier Elemente
    return (item(i).lessthan(item(j)));        // Das knnen die Elemente unter sich ausmachen.
  }

  public boolean gt(int i, int j) {            // Vergleich zweier Elemente
    return lt(j, i);                           // Das knnen die Elemente unter sich ausmachen.
  }

  public boolean eq(int i, int j) {            // Vergleich zweier Elemente
    return (item(i).equal(item(j)));           // Das knnen die Elemente unter sich ausmachen.
  }

  public void exchange(int i, int j) {         // Austausch zweier Elemente
    ielement ele = item(i);                    // c = a; - Das typische Prinzip
    item(i, item(j));                          // a = b;
    item(j, ele);                              // b = c;
  }

  public void move(int i, int j) {             // Verschieben eines Elementes
    if (i != j) {
      if (i < j) { moveup  (i, j); }           // move right/up
      else       { movedown(i, j); }           // move left/down
    }
  }

  protected void moveup(int i, int j) {       // move fr i < j
    for (int k=i; k<j; k++) {                 // Das ganze geht auch doppelt so schnell (ohne exchange),
      exchange(k, k+1);                       // sieht dann aber unbersichtlicher aus...
    }
  }

  protected void movedown(int i, int j) {     // move fr i >= j
    for (int k=i; k>j; k--) {                 // ... und die Laufzeit ist eh schon schlecht genug,
      exchange(k, k-1);                       // da fllt das garnicht mehr auf :-)
    }
  }

  protected void item(int i, ielement ele) { // Element setzen
    if ((i >= 0) && (i < length)) {          // Ist der Index gltig?
      if (ele != null) {                     // Ist das Element gltig?
        items[i] = ele;                      // Ja, dann zuweisen.
      } else {
        throw new NoSuchElementException("\"null\" ist kein gltiges Element!");
      }
    } else {
      throw new IndexOutOfBoundsException("Index \"" + i + "\" liegt nicht in <0.." + (length-1) + ">!");
    }
  }

  protected ielement item(int i) {           // Element ermitteln
    if ((i >= 0) && (i < length)) {          // Ist Index gltig?
      return items[i];                       // Ja, dann holen und zurckgeben.
    } else {
      throw new IndexOutOfBoundsException("Index \"" + i + "\" liegt nicht in <0.." + (length-1) + ">!");
    }
  }

  protected void append(ielement ele) {        // Feld um ein Element erweitern (am Ende)
    if (length < items.length) {               // Kann Feld noch erweitert werden?
      length++;                                // Lnge um eins erhhen
      item(length-1, ele);                     // und Element reinsetzen.
    } else {
      throw new IndexOutOfBoundsException("Feld besitzt bereits "+length()+" Elemente!");
    }
  }

  public void delete() {                       // Feld um ein Element verkleinern (am Ende)
    item(length-1);                            // Das Element interessiert nicht, wohl aber eine evtl. auftretende Exception.
    items[length-1] = null;                    // Referenz auf das Element entfernen
    length--;                                  // und Lnge erniedrigen.
  }

  public void clear() {                        // Alle Elemente lschen
    while (length() > 0) {
      delete();
    }
  }

  public String toString() {                  // Listeninhalt ausgeben
    String s = "";
    for ( int i=0; i<length; i++ ) {
      s += "," + item(i).toString();
    }
    if (s.length() > 0) {s = s.substring(1); }
    return ("(" + s + ")");
  }
}

