// Bezeichnung: sortbucket - Eine Liste aller enthaltenen Sortieralgorithmen.
// Autor      : Peter Weigel, Andreas Brotzmann

// Hat nichts mit bucketsort zu tun!

package pefas.sort;

public class sortbucket {
  protected static String prefix = "pefas.sort.";
  protected static String[][] info;
  static {
    String[][] sa =  {{"trippelsort", "1", "Variante 1"},
                      {"trippelsort", "2", "Variante 2"},
                      {"bubblesort" , "1", "Variante 1"},
                      {"bubblesort" , "3", "Variante 3"},
                      {"bubblesort" , "4", "Variante 4"},
                      {"bubblesort" , "5", "Variante 5 (1p)"},
                      {"shakersort" , "1", "Variante 1"},
                      {"shakersort" , "2", "Variante 2"},
                      {"shakersort" , "3", "Variante 3"},
                      {"ripplesort" , "1", "Variante 1"},
                      {"oetsort"    , "1", "Variante 1"},
                      {"selectsort" , "1", "Variante 1"},
                      {"selectsort" , "3", "Variante 3"},
                      {"selectsort" , "4", "Variante 4"},
                      {"simplesort" , "1", "Variante 1"},
                      {"simplesort" , "2", "Variante 2"},
                      {"simplesort" , "3", "Variante 3"},
                      {"simplesort" , "4", "Variante 4 (1p)"},
                      {"simplesort" , "5", "Variante 5 (2p)"},
                      {"insertsort" , "1", "Variante 1"},
                      {"insertsort" , "2", "Variante 2"},
                      {"insertsort" , "6", "Variante 6 (1p)"},
                      {"plaselsort" , "1", "Variante 1"},
                      {"plaselsort" , "2", "Variante 2"},
                      {"jumpsort"   , "1", "Variante 1"},
                      {"partitsort" , "1", "Variante 1"},
                      {"shearsort"  , "1", "Variante 1"},
                      {"shearsort"  , "2", "Variante 2"},
                      {"shellsort"  , "2", "Variante \"Shell\""},
                      {"shellsort"  , "3", "Variante \"Papernov-Stasevich\""},
                      {"shellsort"  , "4", "Variante \"Knuth\""},
                      {"shellsort"  , "5", "Variante \"11/5\""},
                      {"combsort"   , "2", "Variante 2"},
                      {"combsort"   , "3", "Variante 3 (2p)"},
                      {"bitonicsort", "1", "Variante \"depth first\""},
                      {"bitonicsort", "2", "Variante \"breadth first\""},
                      {"quicksort"  , "1", "Variante 1"},
                      {"quicksort"  , "2", "Variante \"Tri-Median\""},
                      {"mergesort"  , "15", "Variante 5 (depth first)"},
                      {"mergesort"  , "25", "Variante 5 (breadth first)"},
                      {"mergesort"  , "16", "Variante 6 (depth first)"},
                      {"mergesort"  , "26", "Variante 6 (breadth first)"},
                      {"btmsort"    , "15", "Variante 5 (depth first)"},
                      {"btmsort"    , "25", "Variante 5 (breadth first)"},
                      {"btmsort"    , "16", "Variante 6 (depth first)"},
                      {"btmsort"    , "26", "Variante 6 (breadth first)"},
                      {"heapsort"   , "2", "Variante \"binary heap\""},
                      {"heapsort"   , "3", "Variante \"trinary heap\""},
                      {"avlsort"    , "3", "Variante 3"},
                      {"bsort"      , "3", "Variante 3"},
                      {"mssort"     , "1", "Variante 1"}};
    info = sa;
  }

  public static int length() {      // Anzahl verschiedener untersttzter Sortieralgorithmen
    return info.length;
  }

  public static Class getClass(int i) {                 // Klasse ber den Index finden
    try {
      return Class.forName(prefix + info[i][0] + info[i][1]);
    } catch (ClassNotFoundException e) {return null;}
  }

  public static isort newInstance(int i) {              // Instanz ber den Index erzeugen
    try { try {
      return (isort) (getClass(i).newInstance());
    } catch (IllegalAccessException e0) {return null;}
    } catch (InstantiationException e1) {return null;}
  }

  public static String getName(int i) {                 // Name ermitteln (langer Name)
    String s = info[i][0] + ", " + info[i][2];
    s = s.substring(0,1).toUpperCase() + s.substring(1);// Erster Buchstabe gro
    return s;
  }

  public static String[] getNames() { // Name der Sortieralgorithmen als Liste
    String[] n0 = new String[info.length];
    int j = 0;
    for (int i=0; i<length();i++) {
      if ((i==0) || (!info[i][0].equals(info[i-1][0]))) {
        n0[j] = info[i][0];
        n0[j] = n0[j].substring(0,1).toUpperCase() + n0[j].substring(1);// Erster Buchstabe gro
        j++;
      }
    }
    String[] n1 = new String[j];
    for (int i=0; i<j;i++) { n1[i] = n0[i]; }
    return n1;
  }

  public static String[] getVariants(String s) { // Varianten eines Algorithmus als Liste
    String[] v0 = new String[info.length];
    int j = 0;
    for (int i=0; i<length();i++) {
      if (info[i][0].equalsIgnoreCase(s)) {
        v0[j] = info[i][2];
        j++;
      }
    }
    String[] v1 = new String[j];
    for (int i=0; i<j;i++) { v1[i] = v0[i]; }
    return v1;

  }

  public static Class getClass(String name, String variant) { // Klasse ber Namen und Variante finden
    return getClass(name+", "+variant);
  }

  public static isort newInstance(String name, String variant) { // Instanz ber Name und Variante erzeugen
    return newInstance(name+", "+variant);
  }

  public static Class getClass(String s) {             // Klasse ber Namen finden
    for (int i=0; i<length(); i++) {
      if ((s.equalsIgnoreCase(info[i][0]+info[i][1])) ||
          (s.equalsIgnoreCase(info[i][0]+", "+info[i][2]))) {
         return getClass(i);
      }
    }
    return null;
  }

  public static isort newInstance(String s) {         // Instanz ber den String erzeugen
    try { try {
      return (isort) (getClass(s).newInstance());
    } catch (IllegalAccessException e0) {return null;}
    } catch (InstantiationException e1) {return null;}
  }

}

