/* Merge-Splitting-Sort */
/* mit Quicksort und Odd-Even-Transposition-Sort */

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <sys/times.h>
#include <mpi.h>
#include <unistd.h>

#define swap(a,b) {long s_;s_=a;a=b;b=s_;}

long* B; // Temporres Feld

long partition(long* A, long n) { // Partition: Wird intern von Quicksort bentigt
  long x = A[0];
  long i = -1;
  long j =  n;
  while (1) {
    do {j--;} while (A[j] > x);
    do {i++;} while (A[i] < x);
    if (i < j) swap(A[i], A[j])
    else       return j+1;
  }
}

void quicksort(long* A, long n) {  // Quicksort: Sequentielle Sortierung
  long q;
  if (n > 1) {
    q = partition(A, n);
    quicksort(&A[0], q);
    quicksort(&A[q], n-q);
  }
}

void merge(long* A, long n, long q) { // Merge: Zwei Teilfelder in O(n) mischen
  long i, j, k;                       // A Feld, n Feldgre, q Gre des linken Teilfeldes
  for (i=0;i<q;i++) {
    B[i] = A[i];
  }
  for (i=q;i<n;i++) {
    B[n-i+q-1] = A[i];
  }
  i = 0;
  j = n-1;
  for (k=0;k<n;k++) {
    if (B[i] <= B[j]) {
      A[k]= B[i];
      i++;
    } else {
      A[k]= B[j];
      j--;
    }
  }
}

void shift(long* A, long n, long d) { // Shift: Daten verschieben, wird intern von mssort2 verwendet
  long i;                             // d<0 nach Links, d>0 nach Rechts
  if (d < 0) {
    for (i=0;i<n+d;i++) {
      A[i] = A[i-d];
    }
  } else {
    for (i=n-1;i>=d;i--) {
      A[i] = A[i-d];
    }
  }
}

void mssort1(long* A, long n, int p, int r) {  // Merge-Splitting-Sort, Variante 1
  MPI_Status s;                                // Ein Prozess sendet sein Teilfeld zum Nachbarn
  MPI_Request req1, req0 = MPI_REQUEST_NULL;   // der Nachbar mergt mit seinem eigenen
  int i;                                       // und schickt einen Teil wieder zurck

  quicksort(A, n);

  for (i=0;i<p;i++) {
    if ((r+i)%2 == 0) {
      if (r < p-1) {
        MPI_Wait(&req0, &s);                                         // letzten Sendevorgang korrekt abschlieen
        MPI_Recv (&A[n], n, MPI_LONG, r+1, 0, MPI_COMM_WORLD, &s);   // rechtes Teilfeld empfangen
        merge(A, 2*n, n);                                            // Merge
        MPI_Isend(&A[n], n, MPI_LONG, r+1, 0, MPI_COMM_WORLD, &req0);// Splitting
      }
    } else {
      if (r > 0) {
        MPI_Isend(&A[0], n, MPI_LONG, r-1, 0, MPI_COMM_WORLD, &req1); // eigenes Teilfeld wegschicken
        MPI_Recv (&A[0], n, MPI_LONG, r-1, 0, MPI_COMM_WORLD, &s);    // eigenes Teilfeld zurckholen
        MPI_Wait(&req1, &s);                                          // letzten Sendevorgang korrekt abschlieen
      }
    }
  }
  MPI_Wait(&req0, &s); // Letzter Datenaustausch mu abgeschlossen sein
}

/*void mssort2(long* A, long n, int p, int r) {  // Merge-Splitting-Sort, Variante 2
  MPI_Status  s;                               // Alle Prozesse senden Teilfeld zum Nachbarn und empfangen eines vom Nachbarn
  MPI_Request req;                             // Alle Prozesse mergen fremdes Teilfeld mit dem eigenen
  int i;                                       // und verwerfen eine Hlfte
                                               // a: weniger Speicherplatz, mehr Aufwand
  quicksort(A, n);

  for (i=0;i<p;i++) {
    if ((r+i)%2 == 0) {
      if (r < p-1) {
        MPI_Isend(&A[0], n, MPI_LONG, r+1, 0, MPI_COMM_WORLD, &req0);// eigenes Teilfeld wegschicken
        MPI_Recv (&A[n], n, MPI_LONG, r+1, 0, MPI_COMM_WORLD, &s);  // rechtes Teilfeld empfangen
        MPI_Wait(&req0, &s);
        merge(A, 2*n, n);                                           // Merge und rechten Teil verwerfen
      }
    } else {
      if (r > 0) {
        MPI_Isend(&A[0], n, MPI_LONG, r-1, 0, MPI_COMM_WORLD, &req0);// eigenes Teilfeld wegschicken
        MPI_Recv (&A[n], n, MPI_LONG, r-1, 0, MPI_COMM_WORLD, &s);  // linkes Teilfeld empfangen
        MPI_Wait(&req0, &s);
        merge(A, 2*n, n);                                           // Merge
        shift(A, 2*n, -n);                                          // linkes Teilfeld verwerfen
      }
    }
  }
}*/

void mssort2(long* A, long n, int p, int r) {  // Merge-Splitting-Sort, Variante 2
  MPI_Status  s;                               // Alle Prozesse senden Teilfeld zum Nachbarn und empfangen eines vom Nachbarn
  MPI_Request req0;                            // Alle Prozesse mergen fremdes Teilfeld mit dem eigenen
  int i;                                       // und verwerfen eine Hlfte
                                               // b: mehr Speicherplatz, weniger Aufwand
  quicksort(A, n);
  shift(A, 2*n, n); // Unser Hauptfeld ist A[n..2n-1]

  for (i=0;i<p;i++) {
    if ((r+i)%2 == 0) {
      if (r < p-1) {
        MPI_Isend(&A[n]  , n, MPI_LONG, r+1, 0, MPI_COMM_WORLD, &req0);// eigenes Teilfeld wegschicken
        MPI_Recv (&A[2*n], n, MPI_LONG, r+1, 0, MPI_COMM_WORLD, &s);  // rechtes Teilfeld empfangen
        MPI_Wait(&req0, &s);
        merge(&A[n], 2*n, n);                                         // Merge und rechten Teil verwerfen
      }
    } else {
      if (r > 0) {
        MPI_Isend(&A[n], n, MPI_LONG, r-1, 0, MPI_COMM_WORLD, &req0);// eigenes Teilfeld wegschicken
        MPI_Recv (&A[0], n, MPI_LONG, r-1, 0, MPI_COMM_WORLD, &s);  // linkes Teilfeld empfangen
        MPI_Wait(&req0, &s);
        merge(&A[0], 2*n, n);                                       // Merge und linken Teil verwerfen
      }
    }
  }
  shift(A, 2*n, -n);
}

void mssort3(long* A, long n, int p, int r) {  // Merge-Splitting-Sort, Variante 3
  MPI_Status  s;                               // Jeder Prozess hlt immer zwei Teilfelder
  MPI_Request req0 = MPI_REQUEST_NULL;         // Dadurch hat jeder immer was zu tun
  MPI_Request req1 = MPI_REQUEST_NULL;
  MPI_Request req2 = MPI_REQUEST_NULL;
  int i;

  n = (long) (n/2);

  quicksort(&A[0], n);
  quicksort(&A[n], n);

  for (i=0;i<2*p;i++) {
    if (i%2 == 0) {
      MPI_Wait(&req0, &s);                                            // letzten Sendevorgang korrekt abschlieen
      MPI_Wait(&req1, &s);                                            // das mu sein!
      merge(A, 2*n, n);                                               // Merge (die Teilfelder haben wir ja bereits)
    } else {
      if (r > 0) {
        MPI_Isend(&A[0], n, MPI_LONG, r-1, 0, MPI_COMM_WORLD, &req0); // linkes Teilfeld wegschicken
        MPI_Irecv(&A[0], n, MPI_LONG, r-1, 0, MPI_COMM_WORLD, &req1); // linkes Teilfeld zurckholen
      }
      if (r < p-1) {
        MPI_Wait(&req2, &s);                                          // letzten Sendevorgang korrekt abschlieen
        MPI_Recv(&A[2*n], n, MPI_LONG, r+1, 0, MPI_COMM_WORLD, &s);   // rechtes Teilfeld holen
        merge(&A[n], 2*n, n);                                         // Merge
        MPI_Isend(&A[2*n], n, MPI_LONG, r+1, 0, MPI_COMM_WORLD, &req2);// Split
      }
    }
  }
  MPI_Wait(&req0, &s); // Letzter Datenaustausch mu abgeschlossen sein
  MPI_Wait(&req1, &s);
  MPI_Wait(&req2, &s);
}

void randfill(long* A, long n) {  // Randfill: Feld mit Zufallszahlen fllen
  long i;
  for (i=0;i<n;i++) {
    A[i] = rand(); // Feld A fllen, Werte zwischen 0..RAND_MAX
  }
}

int check(long* A, long n, int p, int r) {  // Check (search_for_errors): berprfung, ob Sortierung korrekt war
  long e, i;
  int error = 0;
  MPI_Request req;
  MPI_Status  s;

  if (r < p-1) {
    MPI_Isend(&A[n-1], 1, MPI_LONG, r+1, 0, MPI_COMM_WORLD, &req);      // rechtes Element wegschicken
  }
  if (r > 0) {
    MPI_Recv(&e, 1, MPI_LONG, r-1, 0, MPI_COMM_WORLD, &s);  // rechtes Element des linken Nachbarn holen
    if (e > A[0]) { // Fehler gefunden
      error = 1;
    }
  }

  if (!error) {                                // Bubble-Test aller lokalen Elemente
    for (i=0;i<n-1;i++) {
      if (A[i] > A[i+1]) { // Fehler gefunden
        error = 1;
      }
    }
  }

  MPI_Reduce(&error, &error, 1, MPI_INT, MPI_SUM, 0, MPI_COMM_WORLD); // Check-Ergebnis von allen Prozessen einsammeln

  return error;
}


// Zeitmessung

double st_r, et_r;       // Variablen fr Zeitstempel
struct tms st_us, et_us; // dito
long   ticks;
float  tr, tu, ts;

void time_start() {
  st_r  = MPI_Wtime();
  ticks = sysconf(_SC_CLK_TCK);
  times(&st_us);
}

void time_stop() {
  et_r = MPI_Wtime();
  times(&et_us);
  tr  = et_r - st_r;
  tu = (et_us.tms_utime - st_us.tms_utime) / (float) ticks;
  ts = (et_us.tms_stime - st_us.tms_stime) / (float) ticks;
}


main (int argc, char* argv[]) {
  int n, m, v;       // Elemente global und lokal, Variante
  int r, p;          // Prozessnummer und - anzahl
  long* A;           // lokales Teilfeld

  MPI_Init(&argc, &argv);

  MPI_Comm_rank(MPI_COMM_WORLD, &r);  // Prozessnummer holen
  MPI_Comm_size(MPI_COMM_WORLD, &p); // Prozessanzahl holen

  if (r == 0) printf("Merge-Splitting-Sort (parallel), in drei Varianten\n\n");

  if (argc != 3) {
    if (r == 0) printf("Aufruf: %s \"Anzahl der Elemente\":[1..] \"Variante\":[1..3]\n",argv[0]);
  } else {

  n = atoi(argv[1]);       // n als erstes Argument
  m = (n+2*p-1)/2/p;       // Elemente pro Prozess berechnen, dabei aufrunden
  m = m*2;                 // "div 2" war nur fr Variante 3
  n = m*p;                 // n korregieren
  v = atoi(argv[2]);       // v als zweites Argument

  if (r == 0) {
    printf("Elemente (n)......%d\n", n);
    printf("Prozesse (p)......%d\n", p);
    printf("Variante (v)......%d\n", v);
    printf("Initialisierung...");
  }

  MPI_Barrier(MPI_COMM_WORLD);

  {
    int k[2];
         if (v == 1) {k[0]=2*m;   k[1]=2*m;}       /* Alternative: k[0]=k[1]=3*m */
  /*else if (v == 2) {k[0]=2*m;   k[1]=2*m;} */
    else if (v == 2) {k[0]=3*m;   k[1]=2*m;}
    else             {k[0]=3*m/2; k[1]=m;}

    A = (long*) malloc(k[0]*sizeof(long)); // Speicher fr Elemente belegen
    B = (long*) malloc(k[1]*sizeof(long)); // Temporren Speicher fr Elemente belegen (siehe Merge)
  }

  randfill(A, m);              // Feld fllen

  MPI_Barrier(MPI_COMM_WORLD); // Zeitmessung - Start

  if (r == 0) printf("OK\nSortierung........");

  if (r == 0) {
    time_start();
  }

  if (v == 1)      {mssort1(A, m, p, r);}          // Elemente sortieren
  else if (v == 2) {mssort2(A, m, p, r);}
  else {mssort3(A, m, p, r);}

  MPI_Barrier(MPI_COMM_WORLD); // Zeitmessung - Ende

  if (r == 0) {
    time_stop();
    printf("OK\nLaufzeit (real)...%0.5f Sekunden\n", tr);
    printf("Laufzeit (user)...%0.5f Sekunden\n", tu);
    printf("Laufzeit (system).%0.5f Sekunden\n", ts);
    printf("Analyse...........");
  }

  if ((check(A, m, p, r)) && (r == 0)) {            // Korrektheit prfen
    printf("Sortierung ist fehlerhaft!\n");
  } else if (r == 0) printf("OK\n");

  }

  MPI_Finalize();
}